"""
This module contains varous helper functions related to color.
"""


import re


# fmt: off
NAMED_COLORS = {
    "white":   (0xFF, 0xFF, 0xFF),
    "silver":  (0xC0, 0xC0, 0xC0),
    "gray":    (0x80, 0x80, 0x80),
    "black":   (0x00, 0x00, 0x00),
    "red":     (0xFF, 0x00, 0x00),
    "maroon":  (0x80, 0x00, 0x00),
    "yellow":  (0xFF, 0xFF, 0x00),
    "olive":   (0x80, 0x80, 0x00),
    "lime":    (0x00, 0xFF, 0x00),
    "green":   (0x00, 0x80, 0x00),
    "aqua":    (0x00, 0xFF, 0xFF),
    "teal":    (0x00, 0x80, 0x80),
    "blue":    (0x00, 0x00, 0xFF),
    "navy":    (0x00, 0x00, 0x80),
    "fuchsia": (0xFF, 0x00, 0xFF),
    "purple":  (0x80, 0x00, 0x80),
}
# fmt: on


def is_color(string):
    """Checks if the given string is a valid color.

    :param str string: The string to check.
    :rtype: bool

    >>> is_color("#FF0000") # hexadecimal colors are supported
    True
    >>> is_color("#F00")  # short format allowed
    True
    >>> is_color("FF0000")  # "#" is optionnal
    True
    >>> is_color("#ff0000")  # case insensitive
    True
    >>> is_color("#FF00")  # not a valid color
    False
    >>> is_color("red")  # named color are supported
    True
    >>> is_color("RED")  # named color are case insensitive
    True
    >>> is_color("hello")  # not a valid color name
    False
    """
    if string.lower() in NAMED_COLORS:
        return True
    if re.match(r"^#?[0-9a-f]{3}([0-9a-f]{3})?$", string, re.IGNORECASE):
        return True
    return False


def parse_color_string(color):
    """Converts a color string into an RGB tuple.

    :param str color: The string to convert.
    :return: (r, g, b)

    >>> parse_color_string("#FF0000") # hexadecimal colors are supported
    (255, 0, 0)
    >>> parse_color_string("#F00")  # short format allowed
    (255, 0, 0)
    >>> parse_color_string("FF0000")  # "#" is optionnal
    (255, 0, 0)
    >>> parse_color_string("#ff0000")  # case insensitive
    (255, 0, 0)
    >>> parse_color_string("red")  # named color are supported
    (255, 0, 0)
    >>> parse_color_string("RED")  # named color are case insensitive
    (255, 0, 0)
    """
    # Named color
    if color.lower() in NAMED_COLORS:
        return NAMED_COLORS[color.lower()]

    # #f00 or #ff0000 -> f00 or ff0000
    if color.startswith("#"):
        color = color[1:]

    # f00 -> ff0000
    if len(color) == 3:
        color = color[0] * 2 + color[1] * 2 + color[2] * 2  # noqa

    # ff0000 -> (255, 0, 0)
    return (
        int(color[0:2], 16),
        int(color[2:4], 16),
        int(color[4:], 16),
    )



from gi.repository import Gtk

from . import logger
from . import helpers
from .nautilus_terminal import create_or_update_natilus_terminal


class Crowbar(Gtk.EventBox):
    def __init__(self, uri, window):
        super(Crowbar, self).__init__()

        self.uri = uri
        self.nautilus_window = window
        self.nautilus_app = window.get_application()
        self.path = helpers.gvfs_uri_to_path(uri)

        self.connect_after("parent-set", self._on_parent_set)

    def _on_parent_set(self, widget, old_parent):
        if old_parent:
            self.destroy()
            return
        logger.log("Crowbar inserted at %s" % self.path)
        create_or_update_natilus_terminal(self)



from . import logger


NAUTILUS_ACCELS_BACKUP = {}  # {"action_prefix.action_name": ["accel"], ...}


def backup_nautilus_accels(nautilus_app, nautilus_window):
    if NAUTILUS_ACCELS_BACKUP:
        return

    logger.log("backup_nautilus_accels: backuping accels...")

    # app
    for action in ["app.%s" % a for a in nautilus_app.list_actions()]:
        NAUTILUS_ACCELS_BACKUP[action] = nautilus_app.get_accels_for_action(
            action
        )

    # win, slot, view,...
    for action_prefix in nautilus_window.list_action_prefixes():
        # Skip Nautilus Terminal actions
        if action_prefix.startswith("nterm"):
            continue
        for action in [
            "%s.%s" % (action_prefix, a)
            for a in nautilus_window.get_action_group(
                action_prefix
            ).list_actions()
        ]:
            NAUTILUS_ACCELS_BACKUP[
                action
            ] = nautilus_app.get_accels_for_action(action)

    # row
    # TODO get accels from the "row" group (nautilus side bar)


def remove_nautilus_accels(nautilus_app):
    if not NAUTILUS_ACCELS_BACKUP:
        logger.error(
            "remove_nautilus_accels: Accels will not be removed: accels have not been backuped"
        )
        return
    logger.log("remove_nautilus_accels: removing accels...")
    for action in NAUTILUS_ACCELS_BACKUP:
        nautilus_app.set_accels_for_action(action, [])


def restore_nautilus_accels(nautilus_app):
    if not NAUTILUS_ACCELS_BACKUP:
        logger.error(
            "remove_nautilus_accels: Accels cannot be restored: accels have not been backuped"
        )
        return
    logger.log("restore_nautilus_accels: restoring accels...")
    for action, accels in NAUTILUS_ACCELS_BACKUP.items():
        nautilus_app.set_accels_for_action(action, accels)



import os
import pwd

from gi.repository import Gio
import psutil

from . import APPLICATION_ID


def gvfs_uri_to_path(uri):
    gvfs = Gio.Vfs.get_default()
    return gvfs.get_file_for_uri(uri).get_path()


def process_has_child(pid):
    return len(psutil.Process(pid).children()) > 0


def get_process_cwd(pid):
    return psutil.Process(pid).cwd()


def escape_path_for_shell(path):
    return "'%s'" % path.replace("'", "'\\''")


def get_user_default_shell():
    return pwd.getpwuid(os.getuid()).pw_shell


def get_package_schemas_directory():
    """Returns the directory of the package's schemas."""
    return os.path.join(os.path.dirname(os.path.abspath(__file__)), "schemas")


def gsettings_schema_installed(schemas_id):
    """Checks if the schema with the given id is installed or not."""
    default_schemas_source = Gio.SettingsSchemaSource.get_default()
    schema_list = default_schemas_source.list_schemas(True)

    if schemas_id in schema_list.non_relocatable:
        return True

    if schemas_id in schema_list.relocatable:
        return True

    return False


def get_settings(schema_id, schemas_directory=None):
    """Get the settings for the given schema id.

    If the schema is not installed and a schema directory is provided, the
    schemas of the given directory will be loaded.
    """

    if gsettings_schema_installed(schema_id):
        settings = Gio.Settings.new(schema_id)
        return settings

    if schemas_directory:
        source = Gio.SettingsSchemaSource.new_from_directory(
            schemas_directory, Gio.SettingsSchemaSource.get_default(), True
        )
        schema = source.lookup(schema_id, True)
        settings = Gio.Settings.new_full(schema, None, None)
        return settings


def set_all_settings(settings):
    """Sets a value (the default one if not modified) for each setting.
    This allow settings to be visible in dconf-editor even if the schema
    is not installed.
    """
    for key in settings.list_keys():
        settings.set_value(key, settings.get_value(key))


def get_application_settings():
    """Get Nautilus Terminal settings"""
    return get_settings(APPLICATION_ID, get_package_schemas_directory())



import os


DEBUG = "NAUTILUS_TERMINAL_DEBUG" in os.environ


def log(*args):
    if not DEBUG:
        return
    print(
        "\x1B[1;34m[Nautilus Terminal]\x1B[36m[ LOG]\x1B[0m %s"
        % " ".join([str(item) for item in args])
    )


def warn(*args):
    if not DEBUG:
        return
    print(
        "\x1B[1;34m[Nautilus Terminal]\x1B[33m[WARN]\x1B[0m %s"
        % " ".join([str(item) for item in args])
    )


def error(*args):
    if not DEBUG:
        return
    print(
        "\x1B[1;34m[Nautilus Terminal]\x1B[31m[ ERR]\x1B[0m %s"
        % " ".join([str(item) for item in args])
    )



import os

from gi.repository import Gtk
from gi.repository import GdkPixbuf

from . import APPLICATION_NAME
from . import VERSION


def find_data_path(path):
    root = os.path.abspath(os.path.dirname(__file__))
    return os.path.join(root, path)


class AboutDialog(Gtk.AboutDialog):
    def __init__(self, parent=None):
        Gtk.AboutDialog.__init__(
            self,
            parent=parent,
            program_name=APPLICATION_NAME,
            comments="A terminal embedded in Nautilus, the GNOME's file browser",
            version=VERSION,
            copyright="Copyright (c) 2010-2021 Fabien LOISON",
            website_label="github.com/flozz/nautilus-terminal",
            website="https://github.com/flozz/nautilus-terminal",
            license_type=Gtk.License.GPL_3_0,
        )

        logo = GdkPixbuf.Pixbuf.new_from_file(
            find_data_path("images/nautilus-terminal-logo.svg")
        )
        self.set_logo(logo)



import os
import shutil
import subprocess


XDG_DATA_DIR = os.environ.get("XDG_DATA_DIR", "/usr/share")
XDG_DATA_HOME = os.environ.get(
    "XDG_DATA_HOME", os.path.expanduser("~/.local/share")
)
ROOT = os.path.abspath(os.path.dirname(__file__))

EXTENSION_FILE = "nautilus_terminal_extension.py"
SYSTEM_EXTENSION_DIR = os.path.join(XDG_DATA_DIR, "nautilus-python/extensions")
USER_EXTENSION_DIR = os.path.join(XDG_DATA_HOME, "nautilus-python/extensions")

GLIB_SCHEMA_FILE = "org.flozz.nautilus-terminal.gschema.xml"
GLIB_SCHEMA_SOURCE = os.path.join(ROOT, "schemas", GLIB_SCHEMA_FILE)
SYSTEM_GLIB_SCHEMA_DIR = os.path.join(XDG_DATA_DIR, "glib-2.0/schemas")
USER_GLIB_SCHEMA_DIR = os.path.join(XDG_DATA_HOME, "glib-2.0/schemas")
GLIB_COMPILE_SCHEMA = "/usr/bin/glib-compile-schemas"


def is_packaged():
    return not os.path.isfile(os.path.join(ROOT, "not_packaged.py"))


def is_system_extension_installed():
    return os.path.isfile(
        os.path.join(SYSTEM_EXTENSION_DIR, EXTENSION_FILE)
    ) or os.path.isfile(
        os.path.join(SYSTEM_EXTENSION_DIR, EXTENSION_FILE + "c")  # .pyc
    )


def is_user_extension_installed():
    return os.path.isfile(
        os.path.join(USER_EXTENSION_DIR, EXTENSION_FILE)
    ) or os.path.isfile(
        os.path.join(USER_EXTENSION_DIR, EXTENSION_FILE + "c")  # .pyc
    )


def is_nautilus_python_installed():
    paths = [
        "/usr/lib/nautilus/extensions-3.0/libnautilus-python.so",
        "/usr/lib64/nautilus/extensions-3.0/libnautilus-python.so",
        "/usr/lib/x86_64-linux-gnu/nautilus/extensions-3.0/libnautilus-python.so",
    ]

    for path in paths:
        if os.path.isfile(path):
            return True

    return False


def is_glib_compile_schema_installed():
    return os.path.isfile(GLIB_COMPILE_SCHEMA)


def install_system():
    """Installs the Nautilus extension for a system-wide installation.

    .. WARNING::

       This must be run as root!
    """
    # Copy extension
    if not os.path.isdir(SYSTEM_EXTENSION_DIR):
        os.makedirs(SYSTEM_EXTENSION_DIR)
    shutil.copy(
        os.path.join(ROOT, EXTENSION_FILE),
        os.path.join(SYSTEM_EXTENSION_DIR, EXTENSION_FILE),
    )
    # Copy schemas
    if not os.path.isdir(SYSTEM_GLIB_SCHEMA_DIR):
        os.makedirs(SYSTEM_GLIB_SCHEMA_DIR)
    shutil.copy(
        GLIB_SCHEMA_SOURCE,
        os.path.join(SYSTEM_GLIB_SCHEMA_DIR, GLIB_SCHEMA_FILE),
    )
    # Compile schemas
    if is_glib_compile_schema_installed():
        subprocess.call([GLIB_COMPILE_SCHEMA, SYSTEM_GLIB_SCHEMA_DIR])
        print("GLib schema successfully compiled.")
    else:
        print(
            "GLib schema cannot be compiled. Please install GLib schema compiler and run the following command (as root):"
        )
        print(" ".join([GLIB_COMPILE_SCHEMA, SYSTEM_GLIB_SCHEMA_DIR]))
    #
    print("Nautilus Terminal extension successfully installed on the system.")


def uninstall_system():
    """Remove the Nautilus extension for a system-wide installation.

    .. WARNING::

       This must be run as root!
    """
    files = [
        os.path.join(SYSTEM_EXTENSION_DIR, EXTENSION_FILE),
        os.path.join(SYSTEM_EXTENSION_DIR, EXTENSION_FILE + "c"),  # .pyc
        os.path.join(SYSTEM_GLIB_SCHEMA_DIR, GLIB_SCHEMA_FILE),
    ]
    for file_ in files:
        if os.path.isfile(file_):
            os.remove(file_)
    if is_glib_compile_schema_installed():
        try:
            subprocess.call([GLIB_COMPILE_SCHEMA, SYSTEM_GLIB_SCHEMA_DIR])
        except Exception:
            print("An error occured while trying to recompile glib schemas")
    print(
        "Nautilus Terminal extension successfully uninstalled from the system."
    )


def install_user():
    """Installs the Nautilus extension for the current user.

    .. WARNING::

       This must be run as a regular user!
    """
    # Copy extension
    if not os.path.isdir(USER_EXTENSION_DIR):
        os.makedirs(USER_EXTENSION_DIR)
    shutil.copy(
        os.path.join(ROOT, EXTENSION_FILE),
        os.path.join(USER_EXTENSION_DIR, EXTENSION_FILE),
    )
    # Copy schemas
    if not os.path.isdir(USER_GLIB_SCHEMA_DIR):
        os.makedirs(USER_GLIB_SCHEMA_DIR)
    shutil.copy(
        GLIB_SCHEMA_SOURCE,
        os.path.join(USER_GLIB_SCHEMA_DIR, GLIB_SCHEMA_FILE),
    )
    # Compile schemas
    if is_glib_compile_schema_installed():
        subprocess.call([GLIB_COMPILE_SCHEMA, USER_GLIB_SCHEMA_DIR])
        print("GLib schema successfully compiled.")
    else:
        print(
            "GLib schema cannot be compiled. Please install GLib schema compiler and run the following command:"
        )
        print(" ".join([GLIB_COMPILE_SCHEMA, USER_GLIB_SCHEMA_DIR]))
    #
    print(
        "Nautilus Terminal extension successfully installed on the current user."
    )


def uninstall_user():
    """Remove the Nautilus extension for the current user.

    .. WARNING::

       This must be run as a regular user!
    """
    files = [
        os.path.join(USER_EXTENSION_DIR, EXTENSION_FILE),
        os.path.join(USER_EXTENSION_DIR, EXTENSION_FILE + "c"),  # .pyc
        os.path.join(USER_GLIB_SCHEMA_DIR, GLIB_SCHEMA_FILE),
    ]
    for file_ in files:
        if os.path.isfile(file_):
            os.remove(file_)
    if is_glib_compile_schema_installed():
        try:
            subprocess.call([GLIB_COMPILE_SCHEMA, USER_GLIB_SCHEMA_DIR])
        except Exception:
            print("An error occured while trying to recompile glib schemas")
    print(
        "Nautilus Terminal extension successfully uninstalled from the current user."
    )



import os
import sys
import argparse

from . import VERSION
from .debug import get_debug_info
from .install_nautilus_extension import is_system_extension_installed
from .install_nautilus_extension import is_user_extension_installed
from .install_nautilus_extension import is_nautilus_python_installed
from .install_nautilus_extension import install_system
from .install_nautilus_extension import uninstall_system
from .install_nautilus_extension import install_user
from .install_nautilus_extension import uninstall_user
from .install_nautilus_extension import is_packaged


_EPILOG = """
Please report any bug on Github: https://github.com/flozz/nautilus-terminal/issues
"""


class PrintDebugAction(argparse.Action):
    """Prints debug informations and exit."""

    def __call__(self, parser, namespace, value, option_string=None):
        print(get_debug_info())
        sys.exit(0)


class CheckExtensionAction(argparse.Action):
    """Check if the Nautilus extension is properly installed and exit."""

    def __call__(self, parser, namespace, value, option_string=None):
        if os.getuid() == 0:
            print(
                "E: You must run nautilus-terminal as regular user to perform an installation check."
            )
            sys.exit(1)

        result = ""
        retcode = 0

        result += "\x1B[1mNautilus Python:\x1B[0m "
        if is_nautilus_python_installed():
            result += "\x1B[1;32mInstalled\x1B[0m\n"
        else:
            retcode = 1
            result += "\x1B[1;31mAbsent\x1B[0m\n"
            result += "    Please install Nautilus Python. Please read the documentation:\n"
            result += "    https://github.com/flozz/nautilus-terminal\n"

        result += "\x1B[1mNautilus Terminal Extension:\x1B[0m "
        if (
            is_system_extension_installed()
            and not is_user_extension_installed()
        ) or (
            is_user_extension_installed()
            and not is_system_extension_installed()
        ):
            result += "\x1B[1;32mInstalled\x1B[0m\n"
        elif is_system_extension_installed() and is_user_extension_installed():
            retcode = 1
            result += "\x1B[1;31mError\x1B[0m\n"
            result += "    Nautilus Terminal extension is installed twice...\n"
            result += "    Please remove one of the installed extentions using one of the following commands:\n"
            result += "    \x1B[1;34mSystem-wide:\x1B[0m sudo nautilus-terminal --uninstall-system\n"
            result += "    \x1B[1;34mCurrent user:\x1B[0m nautilus-terminal --uninstall-user\n"
            result += "    NOTE: you may need to replace the 'nautilus-terminal' command by 'python3 -m nautilus_terminal'."
        else:
            retcode = 1
            result += "\x1B[1;31mAbsent\x1B[0m\n"
            result += "    Please install the Nautilus Extension with one of the following commands:\n"
            result += "    \x1B[1;34mSystem-wide:\x1B[0m sudo nautilus-terminal --install-system\n"
            result += "    \x1B[1;34mCurrent user:\x1B[0m nautilus-terminal --install-user\n"
            result += "    NOTE: you may need to replace the 'nautilus-terminal' command by 'python3 -m nautilus_terminal'."

        print(result)
        sys.exit(retcode)


class InstallSystemAction(argparse.Action):
    """Install Nautilus Terminal extention system-wide and exit."""

    def __call__(self, parser, namespace, value, option_string=None):
        if os.getuid() != 0:
            print(
                "E: You must run nautilus-terminal as root to perform a system-wide installation."
            )
            sys.exit(1)
        install_system()
        sys.exit(0)


class UninstallSystemAction(argparse.Action):
    """Uninstall Nautilus Terminal extention system-wide and exit."""

    def __call__(self, parser, namespace, value, option_string=None):
        if os.getuid() != 0:
            print(
                "E: You must run nautilus-terminal as root to perform a system-wide removal."
            )
            sys.exit(1)
        uninstall_system()
        sys.exit(0)


class InstallUserAction(argparse.Action):
    """Install Nautilus Terminal extention for the current user and exit."""

    def __call__(self, parser, namespace, value, option_string=None):
        if os.getuid() == 0:
            print(
                "E: You must run nautilus-terminal as a regular user to perform the installation."
            )
            sys.exit(1)
        install_user()
        sys.exit(0)


class UninstallUserAction(argparse.Action):
    """Uninstall Nautilus Terminal extention from the current user and exit."""

    def __call__(self, parser, namespace, value, option_string=None):
        if os.getuid() == 0:
            print(
                "E: You must run nautilus-terminal as a regular user to perform the removal."
            )
            sys.exit(1)
        uninstall_user()
        sys.exit(0)


class DisplayPackagedMessageAction(argparse.Action):
    """Display a message for packaged Nautilus Terminal versions when the user
    try to call the "installation fix" options."""

    def __call__(self, parser, namespace, value, option_string=None):
        print("E: You cannot perform this action:")
        print(
            "You installed Nautilus Terminal using a distribution package. "
            "If you encounter an issue, try to reinstall it or contact "
            "its maintainer."
        )
        sys.exit(1)


def main(args=sys.argv[1:]):
    cli_parser = argparse.ArgumentParser(
        prog="nautilus-terminal",
        epilog=_EPILOG,
    )

    cli_parser.add_argument(
        "--version",
        action="version",
        version=VERSION,
    )

    cli_parser.add_argument(
        "--print-debug",
        help="Prints debug informations and exit",
        nargs=0,
        action=PrintDebugAction,
    )

    cli_parser.add_argument(
        "--check-extension",
        help="Check if the Nautilus extension is properly installed and exit",
        nargs=0,
        action=CheckExtensionAction,
    )

    if not is_packaged():
        cli_parser.add_argument(
            "--install-system",
            help="Install Nautilus Terminal extention system-wide and exit",
            nargs=0,
            action=InstallSystemAction,
        )

        cli_parser.add_argument(
            "--uninstall-system",
            help="Uninstall Nautilus Terminal extention system-wide and exit",
            nargs=0,
            action=UninstallSystemAction,
        )

        cli_parser.add_argument(
            "--install-user",
            help="Install Nautilus Terminal extention for the current user and exit",
            nargs=0,
            action=InstallUserAction,
        )

        cli_parser.add_argument(
            "--uninstall-user",
            help="Uninstall Nautilus Terminal extention from the current user and exit.",
            nargs=0,
            action=UninstallUserAction,
        )
    else:
        cli_parser.add_argument(
            "--install-system",
            help=argparse.SUPPRESS,
            nargs=0,
            action=DisplayPackagedMessageAction,
        )

        cli_parser.add_argument(
            "--uninstall-system",
            help=argparse.SUPPRESS,
            nargs=0,
            action=DisplayPackagedMessageAction,
        )

        cli_parser.add_argument(
            "--install-user",
            help=argparse.SUPPRESS,
            nargs=0,
            action=DisplayPackagedMessageAction,
        )

        cli_parser.add_argument(
            "--uninstall-user",
            help=argparse.SUPPRESS,
            nargs=0,
            action=DisplayPackagedMessageAction,
        )

    if len(args) == 0:
        cli_parser.parse_args(["--help"])
    else:
        cli_parser.parse_args(args)


if __name__ == "__main__":
    main()



# This file is a part of the Nautilus Temrinal package

# This file is the Nautilus extension itself, it must be placed
# in one of the following folder to make allow the extension to work:
#
# * /usr/share/nautilus-python/extensions/
# * ~/.local/share/nautilus-python/extensions/

import os
import sys

from gi.repository import GObject, Nautilus


DEBUG = "NAUTILUS_TERMINAL_DEBUG_PACKAGE_PATH" in os.environ


if DEBUG:
    sys.path.insert(0, os.environ["NAUTILUS_TERMINAL_DEBUG_PACKAGE_PATH"])


from nautilus_terminal.crowbar import Crowbar  # noqa


if DEBUG:
    print("\x1B[1;34m#### Starting Nautilus Terminal [DEBUG] ####\x1B[0m")
    print("PYTHON_PATH: %s\n" % ":".join(sys.path))
else:
    print("* Starting Nautilus Terminal")


class NautilusTerminalLocationWidgetProvider(
    GObject.GObject, Nautilus.LocationWidgetProvider
):
    def get_widget(self, uri, window):
        return Crowbar(uri, window)



"""
This file should only be present when Nautilus Terminal is installed from
sources (Github or PyPI).

If you are packaging this application for a Linux distribution, please remove
this file, it will disable all the `--install-*` and `--uninstall-*` options of
the `nautilus-terminal` command.
"""



import sys
import os.path
import platform

from . import VERSION
from .install_nautilus_extension import is_system_extension_installed
from .install_nautilus_extension import is_user_extension_installed
from .install_nautilus_extension import is_nautilus_python_installed
from .install_nautilus_extension import is_glib_compile_schema_installed


def _make_title(text):
    separator = "=" * len(text)
    return "\n%s\n%s\n" % (text.upper(), separator)


def _get_os_info():
    result = _make_title("Operating System")
    if platform.system() == "Linux":
        result += _get_os_linux_info()
    else:
        result += "OS: %s" % platform.system()
    return result


def _get_os_linux_info():
    result = ""
    result += "OS: %s\n" % platform.system()
    result += "Platform: %s\n" % platform.platform()
    result += "Version: %s\n" % platform.version()
    if os.path.isfile("/etc/issue"):
        try:
            with open("/etc/issue", "r") as distro_file:
                distro = distro_file.read().strip()
                result += "Distribution issue: %s\n" % distro
        except Exception:
            pass
    return result


def _get_nautilus_terminal_info():
    result = _make_title("Nautilus Terminal")
    result += "Version: %s\n" % VERSION
    result += "System-wide extension: %s\n" % (
        "Installed" if is_system_extension_installed() else "Absent"
    )
    result += "Current user extension: %s\n" % (
        "Installed" if is_user_extension_installed() else "Absent"
    )
    result += "Installation path: %s\n" % os.path.dirname(
        os.path.abspath(__file__)
    )
    return result


def _get_python_info():
    result = _make_title("Python")
    result += "Python version: %d.%d.%d\n" % sys.version_info[:3]
    return result


def _get_system_dependencies_info():
    result = _make_title("System Dependencies")
    result += "Nautilus Python: %s\n" % (
        "Installed" if is_nautilus_python_installed() else "Absent"
    )
    result += "GLib schemas compiler: %s\n" % (
        "Installed" if is_glib_compile_schema_installed() else "Absent"
    )
    return result


def get_debug_info():
    result = ""
    result += _get_nautilus_terminal_info()
    result += _get_os_info()
    result += _get_python_info()
    result += _get_system_dependencies_info()
    return result



#!/usr/bin/env python
# encoding: UTF-8

import os
from setuptools import setup, find_packages
from setuptools.command.install import install as _install

from nautilus_terminal import VERSION


class install(_install):
    def run(self):
        _install.run(self)

        is_root_user = os.getuid() == 0
        is_system_install = bool(self.root) or is_root_user
        is_user_install = not is_system_install
        data_prefix = "%s%s/share" % (
            (self.root if self.root else ""),
            self.install_base,
        )

        print("Installation parameters:")
        print("  is_root_user: %s" % str(is_root_user))
        print("  is_system_install: %s" % str(is_system_install))
        print("  is_user_install: %s" % str(is_user_install))
        print("  data_prefix: %s" % data_prefix)

        if is_system_install:
            os.environ["XDG_DATA_DIR"] = data_prefix

            from nautilus_terminal.install_nautilus_extension import (
                install_system,
            )

            install_system()
        elif is_user_install:
            from nautilus_terminal.install_nautilus_extension import (
                install_user,
            )

            install_user()


long_description = ""
if os.path.isfile("README.rst"):
    long_description = open("README.rst", "r").read()


setup(
    name="nautilus_terminal",
    version=VERSION,
    description="A terminal embedded in Nautilus, the GNOME's file browser",
    url="https://github.com/flozz/nautilus-terminal",
    license="GPL-3.0",
    long_description=long_description,
    author="Fabien LOISON",
    keywords="nautilus extension terminal gnome",
    platforms=["Linux", "BSD"],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "psutil>=5.6.6",
    ],
    extras_require={
        "dev": [
            "nox",
            "flake8",
            "black",
        ],
    },
    cmdclass={"install": install},
    entry_points={
        "console_scripts": [
            "nautilus-terminal = nautilus_terminal.__main__:main",
        ]
    },
)



import os
import signal
import sys

from . import logger
from . import helpers
from . import color_helpers
from . import nautilus_accels_helpers
from .about_dialog import AboutDialog

import gi

gi.require_version("Vte", "2.91")
from gi.repository import GLib, Gio, Gtk, Gdk, Vte, Pango  # noqa: E402


_EXPAND_WIDGETS = [
    "GtkOverlay",
    "NautilusCanvasView",
    "NautilusViewIconController",
    "NautilusListView",
]
_AUTO_CLEAN_SOFT = 1
_AUTO_CLEAN_HARD = 2


def _vte_terminal_feed_child(vte_terminal, text):
    if sys.version_info.major >= 3:
        text = text.encode("utf-8")
    try:
        # Old call
        return vte_terminal.feed_child(text, len(text) + 1)
    except TypeError:
        # Newer call
        return vte_terminal.feed_child(text)


def _find_nautilus_terminal_vpanel(crowbar):
    widget = crowbar
    while widget:
        if widget.get_name() == "GtkVPaned" and hasattr(
            widget, "_nt_instance"
        ):
            return widget
        widget = widget.get_parent()
    return None


def _find_parent_widget(widget, parent_widget_name):
    while widget:
        if widget.get_name() == parent_widget_name:
            return widget
        widget = widget.get_parent()
    return None


def create_or_update_natilus_terminal(crowbar):
    vpanel = _find_nautilus_terminal_vpanel(crowbar)

    # Nautilus Terminal already inserted in this tab, update its path
    if vpanel:
        logger.log("NautilusTerminal instance found: updating its path...")
        vpanel._nt_instance.change_directory(crowbar.path)
        # may update view
        vpanel._nt_instance.update_ui()
        return vpanel._nt_instance

    # New tab, a new Nautilus Terminal instance must be created
    logger.log(
        "No NautilusTerminal instance found (new tab): creating a new NautilusTerminal..."
    )
    nautilus_window_slot = _find_parent_widget(crowbar, "NautilusWindowSlot")

    if not nautilus_window_slot:
        logger.warn(
            "Unable to locate the NautilusWindowSlot widget: Nautilus Terminal will not be injected!"
        )
        return

    return NautilusTerminal(
        nautilus_window_slot,
        crowbar.nautilus_window,
        crowbar.nautilus_app,
        crowbar.path,
    )


class NautilusTerminal(object):
    def __init__(self, parent_widget, nautilus_window, nautilus_app, cwd):
        self._parent_widget = parent_widget  # NautilusWindowSlot
        self._nautilus_window = nautilus_window
        self._nautilus_app = nautilus_app
        self._vbox = None
        self._cwd = cwd

        self._settings = helpers.get_application_settings()
        # Allows settings to be defined in dconf-editor even if
        # the schema is not installed...
        helpers.set_all_settings(self._settings)

        self._ui_vpanel = None
        self._ui_terminal = None
        self._terminal_requested_visibility = self._settings.get_boolean(
            "default-show-terminal"
        )
        self._terminal_focus_on_init = self._settings.get_boolean(
            "default-focus-terminal"
        )
        self._terminal_bottom = self._settings.get_boolean("terminal-bottom")
        self._auto_clean = self._settings.get_enum("auto-clean")
        self._auto_cut_user_input = self._settings.get_boolean(
            "auto-cut-user-input"
        )
        self._nterm_action_group = None
        self._ntermwin_action_group = None
        self._shell_pid = 0
        self._shell_killed = False

        nautilus_accels_helpers.backup_nautilus_accels(
            nautilus_app, nautilus_window
        )
        self._build_and_inject_ui()
        self._build_actions()
        self._insert_ntermwin_action_group_in_current_window()
        self._build_accels()

        # Set if the terminal should be visible by default.
        # Will spawn the shell automatically if the terminal is visible
        self.set_terminal_visible(
            self._terminal_requested_visibility, self._terminal_focus_on_init
        )

    def change_directory(self, path):
        # "virtual" location (trash:///, network:///,...)
        # No cd & hide the Terminal
        if not path:
            logger.log(
                'NautilusTerminal.change_directory: terminal hidden: navigating to a "virtual" location'
            )
            self._ui_terminal.set_visible(False)
            return

        self._cwd = path

        # Makes the terminal visible again if it was hidden by navigating to a
        # "virtual" location
        if (
            self.get_terminal_visible()
            != self.get_terminal_requested_visibility()
        ):
            self.set_terminal_visible(focus=False)

        # Do not navigate if the terminal is not visible
        if not self.get_terminal_visible():
            logger.log(
                "NautilusTerminal.change_directory: current directory NOT changed to %s (terminal not visible)"
                % path
            )
            return

        # Do not "cd" if the shell's cwd is already the same as the targeted path
        if helpers.get_process_cwd(self._shell_pid) == path:
            return

        logger.log(
            "NautilusTerminal.change_directory: current directory changed to %s"
            % path
        )

        command = " cd %s " % helpers.escape_path_for_shell(self._cwd)
        if self.get_auto_clean() == _AUTO_CLEAN_HARD:
            command += "&& clear "

        self._inject_command(command)

        if self.get_auto_clean() == _AUTO_CLEAN_SOFT:
            self._emit_key_press("l", Gdk.ModifierType.CONTROL_MASK)

    def stash_current_termianl_content(self):
        # move to the end of line in terminal
        self._emit_key_press("e", Gdk.ModifierType.CONTROL_MASK)
        # cut all content in line to the left
        self._emit_key_press("u", Gdk.ModifierType.CONTROL_MASK)

    def get_auto_cut_user_input(self):
        return self._auto_cut_user_input

    def get_auto_clean(self):
        return self._auto_clean

    def get_terminal_requested_visibility(self):
        """Does the user requested the terminal to be visible?

        This state can be different to the terminal widget real state, for
        example the terminal can be requested visible by the user but hidden
        because navigating to the trash)?
        """
        return self._terminal_requested_visibility

    def get_terminal_visible(self):
        """Does the terminal widget is visible?"""
        return self._ui_terminal.get_visible()

    def set_terminal_visible(self, visible=None, focus=True):
        if visible is None:
            visible = self._terminal_requested_visibility

        self._ui_terminal.set_visible(visible)
        self._terminal_requested_visibility = visible

        # Spawn a shell if it is not yet spawned (if the terminal has never
        # been visible before)
        if visible and not self._shell_pid:
            self._spawn_shell()

        # Update the directory as the terminal does not "cd" when it is hidden
        if visible:
            self.change_directory(self._cwd)

        # Focus the terminal
        if visible and focus:
            self._ui_terminal.grab_focus()

    def shell_is_busy(self):
        return helpers.process_has_child(self._shell_pid)

    def _inject_command(self, command):
        # Do not inject the command if the shell has something running in
        if self.shell_is_busy():
            logger.warn(
                "NautilusTerminal._inject_command: command '%s' not injected (shell busy)"
                % command
            )
            return
        logger.log("NautilusTerminal._inject_command: %s" % command)

        # Remove any user inputs before injecting the command
        if self.get_auto_cut_user_input():
            self.stash_current_termianl_content()

        _vte_terminal_feed_child(self._ui_terminal, "%s\n" % command)

    def _emit_key_press(self, key, state=0):
        event = Gdk.Event().new(Gdk.EventType.KEY_PRESS)
        event.state = state
        event.keyval = Gdk.keyval_from_name(key)
        event.window = self._ui_terminal.get_window()
        event.send_event = True
        self._ui_terminal.emit("key-press-event", event)

    def update_ui(self):
        for widget in self._parent_widget:
            if widget.get_name() in _EXPAND_WIDGETS:
                self._parent_widget.remove(widget)
                self._vbox.pack_start(widget, True, True, 0)

    def _build_and_inject_ui(self):
        # GtkPaned (vpanel) injection:
        #
        # To inject our terminal in Nautilus, we first have to empty the
        # NautilusWindowSlot widget (parent widget) of the current view to make
        # some free space to insert a GtkPaned widget. Then we will insert back
        # the Nautilus' widgets we removed in the bottom part of the GtkPaned.

        self._ui_vpanel = Gtk.VPaned(visible=True)
        self._ui_vpanel._nt_instance = self
        self._vbox = Gtk.VBox(visible=True)
        if self._terminal_bottom:
            self._ui_vpanel.pack1(self._vbox, resize=True, shrink=True)
        else:
            self._ui_vpanel.pack2(self._vbox, resize=True, shrink=True)

        for widget in self._parent_widget:
            self._parent_widget.remove(widget)
            expand = widget.get_name() in _EXPAND_WIDGETS
            self._vbox.pack_start(widget, expand, expand, 0)
        self._parent_widget.pack_start(self._ui_vpanel, True, True, 0)

        # Terminal creation:
        #
        # We can now create our VteTerminal and insert it in the top part of
        # our GtkPaned.

        self._ui_terminal = Vte.Terminal()

        settings_font = self._settings.get_string("custom-font")
        self._ui_terminal.set_font(Pango.FontDescription(settings_font))

        self._ui_terminal.connect(
            "child-exited", self._on_terminal_child_existed
        )
        if self._terminal_bottom:
            self._ui_vpanel.pack2(
                self._ui_terminal, resize=False, shrink=False
            )
        else:
            self._ui_vpanel.pack1(
                self._ui_terminal, resize=False, shrink=False
            )

        TERMINAL_CHAR_HEIGHT = self._ui_terminal.get_char_height()
        TERMINAL_BORDER_WIDTH = 1
        TERMINAL_MIN_HEIGHT = self._settings.get_uint("min-terminal-height")

        self._ui_terminal.set_property(
            "height-request",
            TERMINAL_CHAR_HEIGHT * TERMINAL_MIN_HEIGHT
            + TERMINAL_BORDER_WIDTH * 2,
        )

        # Terminal colors

        fg_color = (255, 255, 255)
        bg_color = (0, 0, 0)

        settings_fg_color = self._settings.get_string("foreground-color")
        settings_bg_color = self._settings.get_string("background-color")

        if color_helpers.is_color(settings_fg_color):
            fg_color = color_helpers.parse_color_string(settings_fg_color)

        if color_helpers.is_color(settings_bg_color):
            bg_color = color_helpers.parse_color_string(settings_bg_color)

        self._ui_terminal.set_color_foreground(
            Gdk.RGBA(
                fg_color[0] / 255.0,
                fg_color[1] / 255.0,
                fg_color[2] / 255.0,
                1,
            )
        )

        self._ui_terminal.set_color_background(
            Gdk.RGBA(
                bg_color[0] / 255.0,
                bg_color[1] / 255.0,
                bg_color[2] / 255.0,
                1,
            )
        )

        # File drag & drop support

        self._ui_terminal.drag_dest_set(
            Gtk.DestDefaults.MOTION
            | Gtk.DestDefaults.HIGHLIGHT
            | Gtk.DestDefaults.DROP,
            [Gtk.TargetEntry.new("text/uri-list", 0, 0)],
            Gdk.DragAction.COPY,
        )
        self._ui_terminal.drag_dest_add_uri_targets()
        self._ui_terminal.connect(
            "drag-data-received", self._on_terminal_drag_data_received
        )

        # Disabling Nautilus' accels when the terminal is focused:
        #
        # When the terminal is focused, we MUST remove ALL Nautilus accels to
        # be able to use properly the terminal (else we cannot type some
        # characters like "/", "~",... because the actions of the corresponding
        # accels are called). Of course we have also to restore the accels when
        # the terminal is no more focused.

        self._ui_terminal.connect(
            "focus-in-event", self._on_terminal_focus_in_event
        )
        self._ui_terminal.connect(
            "focus-out-event", self._on_terminal_focus_out_event
        )
        # Register the window-level action group of the currently displayed
        # terminal
        #
        # When the active tab changes, we have to register on the nautilus
        # window the corresponding action group to allow related accels to work
        # properly. The "map" event of the NautilusWindowSlot (our parent
        # widget) is called when its tab become active.

        self._parent_widget.connect(
            "map", self._on_nautilus_window_slot_mapped
        )

        # Kill the shell when the tab is closed:
        #
        # Finally, we have to listen to some events of our parent widget (the
        # NautilusWindowSlot) to guess when the tab is closed in order to
        # kill the shell (else we will have plenty zombie shell running in
        # background).
        #
        # * When the tab (or the window) is closed, the "unrealize" event is
        #   called so we have to kill the shell when the parent widget emit
        #   this event...
        #
        # * BUT when the tab is drag & dropped outside of the window (to open
        #   it in a new window), this event is also emitted... and the
        #   "realize" event is emitted right after. So we have to spawn a new
        #   shell if that happen...

        self._parent_widget.connect(
            "unrealize", self._on_nautilus_window_slot_unrealized
        )
        self._parent_widget.connect(
            "realize", self._on_nautilus_window_slot_realized
        )

        # Terminal Context Menu

        self._ui_menu = Gtk.Menu()

        menu_item = Gtk.ImageMenuItem.new_from_stock("gtk-copy", None)
        menu_item.connect_after(
            "activate", lambda w: self._ui_terminal.copy_clipboard()
        )
        self._ui_menu.add(menu_item)

        menu_item = Gtk.ImageMenuItem.new_from_stock("gtk-paste", None)
        menu_item.connect_after(
            "activate", lambda w: self._ui_terminal.paste_clipboard()
        )
        self._ui_menu.add(menu_item)

        self._ui_menu.add(Gtk.SeparatorMenuItem())

        menu_item = Gtk.ImageMenuItem.new_from_stock("gtk-preferences", None)
        menu_item.connect_after("activate", self._on_menu_preferences_activate)
        self._ui_menu.add(menu_item)

        menu_item = Gtk.ImageMenuItem.new_from_stock("gtk-about", None)
        menu_item.connect_after("activate", self._on_menu_about_activate)
        self._ui_menu.add(menu_item)

        self._ui_menu.show_all()
        self._ui_terminal.connect(
            "button-release-event", self._on_terminal_popup_menu
        )

    def _build_actions(self):
        # nterm action group
        self._nterm_action_group = Gio.SimpleActionGroup()
        self._ui_terminal.insert_action_group(
            "nterm", self._nterm_action_group
        )

        copy_action = Gio.SimpleAction(name="copy")
        copy_action.connect("activate", self._on_nterm_copy_action_activated)
        self._nterm_action_group.add_action(copy_action)

        paste_action = Gio.SimpleAction(name="paste")
        paste_action.connect("activate", self._on_nterm_paste_action_activated)
        self._nterm_action_group.add_action(paste_action)

        # ntermwin action group
        self._ntermwin_action_group = Gio.SimpleActionGroup()

        terminal_visible_action = Gio.SimpleAction(name="terminal-visible")
        terminal_visible_action.connect(
            "activate", self._on_ntermwin_terminal_visible_action_activated
        )
        self._ntermwin_action_group.add_action(terminal_visible_action)

    def _insert_ntermwin_action_group_in_current_window(self):
        self._nautilus_window.insert_action_group(
            "ntermwin", self._ntermwin_action_group
        )

    def _build_accels(self):
        # nterm
        self._nautilus_app.set_accels_for_action(
            "nterm.copy", ["<Primary><Shift>c"]
        )
        self._nautilus_app.set_accels_for_action(
            "nterm.paste", ["<Primary><Shift>v"]
        )

        # ntermwin
        accel = self._settings.get_string("toggle-shortcut")
        self._nautilus_app.set_accels_for_action(
            "ntermwin.terminal-visible", [accel]
        )

    def _spawn_shell(self):
        if self._shell_pid:
            logger.warn(
                "NautilusTerminal._spawn_shell: Cannot spawn a new shell: there is already a shell running."
            )
            return
        shell = helpers.get_user_default_shell()
        if self._settings.get_boolean("use-custom-command"):
            shell = self._settings.get_string("custom-command")
        _, self._shell_pid = self._ui_terminal.spawn_sync(
            Vte.PtyFlags.DEFAULT,
            self._cwd,
            [shell],
            None,
            GLib.SpawnFlags.SEARCH_PATH,
            None,
            None,
        )
        self._shell_killed = False
        logger.log(
            "NautilusTerminal._spawn_shell: Shell spawned (%s), PID: %i."
            % (shell, self._shell_pid)
        )

    def _kill_shell(self):
        if not self._shell_pid:
            logger.warn(
                "NautilusTerminal._kill_shell: Cannot kill the shell: there is no shell to kill..."
            )
            return
        self._shell_killed = True
        try:
            os.kill(self._shell_pid, signal.SIGTERM)
            os.kill(self._shell_pid, signal.SIGKILL)
        except OSError:
            logger.error(
                "NautilusTerminal._kill_shell: An error occured when killing the shell %i"
                % self._shell_pid
            )
            self._shell_pid = 0
        logger.log("Shell %i killed." % self._shell_pid)
        self._shell_pid = 0

    def _on_nautilus_window_slot_mapped(self, widget):
        logger.log("The active tab has changed.")
        self._insert_ntermwin_action_group_in_current_window()

    def _on_nautilus_window_slot_unrealized(self, widget):
        logger.log(
            "The tab have (probably) been closed: killing the shell %i"
            % self._shell_pid
        )
        self._kill_shell()
        self._nautilus_window = None

    def _on_nautilus_window_slot_realized(self, widget):
        logger.log("Oops, the tab have NOT been closed: spawning a new shell")
        self._spawn_shell()
        self._nautilus_window = _find_parent_widget(widget, "NautilusWindow")
        self._insert_ntermwin_action_group_in_current_window()

    def _on_terminal_focus_in_event(self, widget, event):
        nautilus_accels_helpers.remove_nautilus_accels(self._nautilus_app)

    def _on_terminal_focus_out_event(self, widget, event):
        nautilus_accels_helpers.restore_nautilus_accels(self._nautilus_app)

    def _on_terminal_child_existed(self, widget, arg1):
        self._shell_pid = 0
        if not self._shell_killed:
            self._spawn_shell()

    def _on_terminal_drag_data_received(
        self, widget, context, x, y, data, info, time
    ):
        for uri in data.get_uris():
            path = helpers.escape_path_for_shell(helpers.gvfs_uri_to_path(uri))
            _vte_terminal_feed_child(self._ui_terminal, "%s " % path)
        self._ui_terminal.grab_focus()

    def _on_terminal_popup_menu(self, widget, event):
        if event.type == Gdk.EventType.BUTTON_RELEASE and event.button != 3:
            return
        self._ui_menu.popup(None, None, None, None, 3, 0)

    def _on_menu_preferences_activate(self, widget):
        self._inject_command("dconf-editor /org/flozz/nautilus-terminal")

    def _on_menu_about_activate(self, widget):
        about_dialog = AboutDialog(parent=self._nautilus_window)
        about_dialog.run()
        about_dialog.destroy()

    def _on_nterm_copy_action_activated(self, action, parameter):
        logger.log("nterm.copy action activated")
        self._ui_terminal.copy_clipboard()

    def _on_nterm_paste_action_activated(self, action, parameter):
        logger.log("nterm.paste action activated")
        self._ui_terminal.paste_clipboard()

    def _on_ntermwin_terminal_visible_action_activated(
        self, action, parameter
    ):
        logger.log("ntermwin.terminal-visible action activated")
        self.set_terminal_visible(not self.get_terminal_visible())
